#!/usr/bin/env python3

from math import hypot, pi
import unittest

import johnson
import validate_ssn


class TestJohnson(unittest.TestCase):
    target = "johnson.py"
    points = 2.5

    def test_no_jobs(self):
        jobs = {}
        actual = johnson.solve(jobs)
        self.assertListEqual([], actual)

    def test_one_job(self):
        jobs = {17: [5, 2]}
        actual = johnson.solve(jobs)
        self.assertListEqual([17], actual)

    def test_johnson(self):
        msg = "Johnson's rule must be implemented correctly"
        jobs = {1: [5, 2],
                2: [3, 6],
                3: [8, 4],
                4: [10, 7],
                5: [7, 12]}
        actual = johnson.solve(jobs)
        self.assertListEqual(actual, [2, 5, 4, 3, 1], msg)
        jobs = {1: [4, 3],
                2: [7, 2],
                3: [9, 4],
                4: [4, 1]}
        print('next')
        actual = johnson.solve(jobs)
        self.assertListEqual(actual, [3, 1, 2, 4], msg)

    def test_johnson_tie(self):
        jobs = {1: [3, 12],
                2: [6, 6],
                3: [9, 6],
                4: [7, 14],
                5: [12, 8],
                6: [9, 7]}
        actual = johnson.solve(jobs)
        expected1 = [1, 2, 4, 5, 6, 3]
        expected2 = [1, 4, 5, 6, 3, 2]
        expected3 = [1, 4, 5, 6, 2, 3]
        # ties in activity time are broken arbitrarily
        self.assertTrue(actual == expected1 or actual == expected2 or
                        actual == expected3)

    def test_module_docstring(self):
        self.assertTrue(johnson.__doc__, 'module documentation required')
        self.assertTrue(len(johnson.__doc__) > 20,
                        'module documentation cannot be too short')

    def test_docstring(self):
        self.assertTrue(johnson.solve.__doc__, 'docstring is missing')
        self.assertTrue(len(johnson.solve.__doc__) > 30,
                        'docstring is missing')


class TestSSN(unittest.TestCase):
    target = "validate_ssn.py"
    points = 1.5

    #def setUp(self):
        #pass

    def test_module_docstring(self):
        self.assertTrue(validate_ssn.__doc__, 'module documentation required')
        self.assertTrue(len(validate_ssn.__doc__) > 20,
                        'module documentation cannot be too short')

    def test_docstring(self):
        self.assertTrue(validate_ssn.is_valid.__doc__, 'docstring is missing')
        self.assertTrue(len(validate_ssn.is_valid.__doc__) > 30,
                        'docstring is too short')

    def test_valid(self):
        valid_ssn = [4, 3, 3, 8, 2, 0, 0, 6, 8, 1]
        self.assertTrue(validate_ssn.is_valid(valid_ssn))
        valid_ssn = [5, 2, 9, 8, 1, 7, 0, 4, 8, 5]
        self.assertTrue(validate_ssn.is_valid(valid_ssn))
        valid_ssn = [7, 1, 4, 9, 2, 9, 0, 2, 1, 2]
        self.assertTrue(validate_ssn.is_valid(valid_ssn))

    def test_invalid_length(self):
        self.assertTrue(validate_ssn.is_valid([]) is False)
        self.assertTrue(validate_ssn.is_valid([2, 0, 0, 6]) is False)
        too_long = [4, 3, 3, 8, 2, 0, 0, 6, 8, 1, 0]
        self.assertTrue(validate_ssn.is_valid(too_long) is False)

    def test_invalid_calculation(self):
        invalid = [4, 3, 3, 9, 2, 0, 0, 6, 8, 1]
        self.assertTrue(validate_ssn.is_valid(invalid) is False)
        invalid = [5, 3, 3, 8, 2, 0, 0, 6, 8, 1]
        self.assertTrue(validate_ssn.is_valid(invalid) is False)
        invalid = [4, 3, 3, 8, 2, 0, 0, 6, 7, 1]
        self.assertTrue(validate_ssn.is_valid(invalid) is False)
        invalid = [4, 3, 3, 8, 2, 0, 1, 6, 8, 1]
        self.assertTrue(validate_ssn.is_valid(invalid) is False)

    def test_invalid_birthday(self):
        invalid = [6, 1, 4, 7, 1, 7, 1, 3, 1, 1]
        self.assertTrue(validate_ssn.is_valid(invalid) is False,
                        'there is no 13th month')
        invalid = [7, 1, 4, 8, 3, 1, 0, 6, 1, 3]
        self.assertTrue(validate_ssn.is_valid(invalid) is False,
                        '31st of June does not exist')
        invalid = [7, 1, 4, 4, 2, 9, 0, 2, 1, 3]
        self.assertTrue(validate_ssn.is_valid(invalid) is False,
                        'February 29th only exists in leap years')

def grader(result):
    """Return the number of points obtained by the result"""
    total = result.testsRun
    successes = total - len(result.errors) - len(result.failures)
    points = successes / total * result.points
    print(round(points, 2), "points for", result.target)
    return points

if __name__ == "__main__":
    tests = (TestJohnson,
             TestSSN)
    total = 0.0
    max_total = 0.0
    for test in tests:
        suite = unittest.defaultTestLoader.loadTestsFromTestCase(test)
        runner = unittest.TextTestRunner()
        result = runner.run(suite)
        result.target = test.target
        result.points = test.points
        total += grader(result)
        max_total += test.points
    print()
    print("{}/{} points in total".format(total, max_total))
