#!/usr/bin/env python3

"""
Simple GUI application for scheduling jobs on two machines.

Copyright (C) 2013 Gerald Senarclens de Grancy <oss@senarclens.eu>

This module is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
"""


import sys

from PyQt5 import QtWidgets, QtWidgets, QtCore

import johnson

class MainWindow(QtWidgets.QMainWindow):
    """Main Window for Johnson GUI application."""
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)
        self.setWindowTitle("Johnson's Rule")
        widget = CentralWidget(self)
        self.setCentralWidget(widget)
        self.resize(widget.width(), 250)


class CentralWidget(QtWidgets.QWidget):
    """All widgets and their behavior required by the Johnson GUI."""
    def __init__(self, parent=None):
        super().__init__(parent)
        self.layout = QtWidgets.QGridLayout()
        self.table = QtWidgets.QTableWidget()
        self.table.setRowCount(2)
        self.table.setColumnCount(3)
        self.layout.addWidget(self.table, 1, 0, 1, 3)
        self.table.setItem(0, 0, QtWidgets.QTableWidgetItem("Job"))
        self.table.setItem(0, 1, QtWidgets.QTableWidgetItem("Machine 1"))
        self.table.setItem(0, 2, QtWidgets.QTableWidgetItem("Machine 2"))
        self.table.setItem(1, 0, QtWidgets.QTableWidgetItem('1'))
        self.__add_buttons()
        self.__adjust_sizes()
        self.setLayout(self.layout)

    def __add_buttons(self):
        self.buttons = {'exit': QtWidgets.QPushButton('Exit'),
                        'solve': QtWidgets.QPushButton('Solve'),
                        'add': QtWidgets.QPushButton('Add'),
                        'remove': QtWidgets.QPushButton('Remove')}
        self.layout.addWidget(self.buttons['solve'], 0, 0)
        self.layout.addWidget(self.buttons['add'], 0, 1)
        self.layout.addWidget(self.buttons['remove'], 0, 2)
        self.buttons['solve'].clicked.connect(self.__solve)
        self.buttons['add'].clicked.connect(self.__add_job)
        self.buttons['remove'].clicked.connect(
            lambda: self.table.setRowCount(self.table.rowCount() - 1))

    def __add_job(self):
        """
        Add a job.
        """
        self.table.setRowCount(self.table.rowCount() + 1)
        try:
            job_number = int(self.table.item(self.table.rowCount() - 2, 0)
                             .text())
            job_number += 1
            self.table.setItem(job_number, 0,
                               QtWidgets.QTableWidgetItem(str(job_number)))
        except:
            pass

    def __adjust_sizes(self):
        w = self.table.verticalHeader().width() + 20
        for index in range(self.table.columnCount()):
            w += self.table.columnWidth(index)
        self.table.resize(w, self.table.height())
        self.resize(self.table.width(), self.height())

    def __solve(self):
        """
        Collect required data and run the external solver.

        Missing time values are set to 0.0.
        """
        jobs = {}
        for row in range(1, self.table.rowCount()):
            for col in range(self.table.columnCount()):
                try:
                    text = self.table.item(row, col).text()
                except AttributeError as e:
                    text = ''
                if col == 0:
                    job = text
                    assert job not in jobs.keys(), "Jobs must be unique"
                    jobs[job] = []
                else:
                    try:
                        time = float(text)
                    except ValueError as e:
                        time = 0.0
                        self.table.setItem(row, col,
                                           QtWidgets.QTableWidgetItem(str(time)))
                    jobs[job].append(time)
        solution = ""
        for id_ in johnson.solve(jobs):
            print(id_)
            solution += id_ + ", "
            print(solution)
        QtWidgets.QMessageBox.information(self, "Schedule", solution[:-2])

if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    t = MainWindow()
    t.show()
    sys.exit(app.exec_())

